/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <system_settings.h>
#include "$(appName).h"
#include "data.h"
#include "view.h"

static bool _get_display_language_cb(const char *locale, char **display_language);
static bool _get_available_locale_cb(const char *locale);
static void _get_startup_lang(char **locale);
static bool _is_same_string(const char *s1, const char *s2);

/**
 * @brief Hook to take necessary actions before main event loop starts.
 * Initialize UI resources and application's data.
 * If this function returns true, the main loop of application starts.
 * If this function returns false, the application is terminated.
 */
static bool app_create(void *user_data)
{
	char *locale = NULL;

	_get_startup_lang(&locale);

	view_set_callback(_get_display_language_cb);

	if (!view_create(user_data))
		return false;

	view_set_startup_language(locale);

	data_get_available_locale(_get_available_locale_cb);



	return true;
}

/**
 * @brief This callback function is called when another application
 * sends a launch request to the application.
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/**
 * @brief This callback function is called each time
 * the application is completely obscured by another application
 * and becomes invisible to the user.
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/**
 * @brief This callback function is called each time
 * the application becomes visible to the user.
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/**
 * @brief This callback function is called once after the main loop of the application exits.
 */
static void app_terminate(void *user_data)
{
	view_destroy();
}

/**
 * @brief This function will be called when the language is changed.
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LANGUAGE_CHANGED */
	char *locale = NULL;

	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

	if (!locale) {
		dlog_print(DLOG_INFO, LOG_TAG, "Function system_settings_get_value_string() failed!");
		free(locale);
		return;
	}

	view_change_language(locale);
}

/**
 * @brief Main function of the application.
 */
int main(int argc, char *argv[])
{
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	/*
	 * If you want to handle more events,
	 * please check the application life cycle guide.
	 */
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, NULL);

	ret = ui_app_main(argc, argv, &event_callback, NULL);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() failed. err = %d", ret);

	return ret;
}

/**
 * @brief Internal callback function responsible for language name acquisition
 * based on given locale.
 * This callback function is invoked by the view module when the user selects
 * a language to apply.
 * @param[in] locale The localization string used for display language identification.
 * @param[out] display_language The language name obtained from localization string.
 * @return This function returns 'true' if the display language was acquired successfully,
 * otherwise 'false' is returned.
 */
static bool _get_display_language_cb(const char *locale, char **display_language)
{
	if (!display_language)
		return false;

	*display_language = NULL;

	return data_get_locale_display_language(locale, display_language);
}

/**
 * @brief Internal callback function responsible for verification if provided
 * locale is supported by this application.
 * This callback function is invoked by the data module for each enumerated locale.
 * Internally, the function checks whether translation file for given locale exists.
 * If the file exists, then it is assumed that the given locale is supported
 * and its name is added to the view module.
 * @param[in] locale The locale provided for verification against translation file existence.
 * @return This function returns 'true' to continue the enumeration. If 'false' is returned,
 * the available locale enumeration stops.
 */
static bool _get_available_locale_cb(const char *locale)
{
	char fpath[512] = { 0, };
	char *res_path = app_get_resource_path();

	snprintf(fpath, sizeof(fpath), "%slocale/%s/LC_MESSAGES/$(appName).mo", res_path, locale);
	free(res_path);

	/* There is no mapping bewtween language code and locale for some countries, so values for them have to be hardcoded*/
	if (_is_same_string(locale, "az_Latn_AZ"))
		view_add_locale("az_AZ");
	else if (_is_same_string(locale, "kk_Cyrl_KZ"))
		view_add_locale("kk_KZ");
	else if (_is_same_string(locale, "sr_Latn_RS"))
			view_add_locale("sr_RS");
	else if (_is_same_string(locale, "uz_Latn_UZ"))
			view_add_locale("uz_UZ");
	else if (_is_same_string(locale, "zh_Hans_CN"))
			view_add_locale("zh_CN");
	else if (_is_same_string(locale, "zh_Hant_HK"))
			view_add_locale("zh_HK");
	else if (_is_same_string(locale, "zh_Hant_TW"))
			view_add_locale("zh_TW");
	else if (access(fpath, F_OK) != -1) {
		dlog_print(DLOG_INFO, LOG_TAG, "Language supported: %s", locale);
		view_add_locale(locale);
	}

	return true;
}

static void _get_startup_lang(char **locale)
{
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, locale);

	if (!locale) {
		dlog_print(DLOG_INFO, LOG_TAG, "Function system_settings_get_value_string() failed!");
		free(locale);
	}

}

/**
 * @brief Internal function which checks whether two strings are the same.
 * @param[in] s1 The first string.
 * @param[in] s2 The second string.
 * @return This function returns 'true' is both given strings are the same,
 * otherwise 'false' is returned.
 */
static bool _is_same_string(const char *s1, const char *s2)
{
	if (!s1 || !s2)
		return false;

	return (strncmp(s1, s2, strlen(s1)) == 0 && strlen(s1) == strlen(s2));
}
